# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.0.beta4'
date = '2023-03-12'
banner = 'SageMath version 10.0.beta4, Release Date: 2023-03-12'
